package api

import (
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
)

func createDefaultDesign(apictx *ApiSession, shoeId string) (*model.Design, error) {

	shoeid, _ := primitive.ObjectIDFromHex(shoeId)

	option := repo.DocSearchOptions{
		CollectName: repo.CollectionShoes,
		Query:       repo.Map{"_id": shoeid},
	}
	shoe := &model.ShoeMesh{}
	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, shoe)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("款式模型已被删除!")
	}
	uid := apictx.User.ID

	//查询用户默认配置
	u := &model.User{}
	_, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionUser,
		Query:       repo.Map{"_id": uid}, //"editorSet.themes": bson.M{"$elemMatch": bson.M{"id": theme.Id}}}
		Project:     []string{"editorSet"},
	}, u)
	if err != nil {
		return nil, err
	}

	//获取默认环境
	optionEnv := repo.DocSearchOptions{
		CollectName: repo.CollectionEvn3d,
		// Query:       repo.Map{"name": "工作室"},
	}
	useDefaultEnvId := false
	if u.EditorSet != nil && len(u.EditorSet.DefaultEnv3dId) > 0 {
		optionEnv.Query = repo.Map{"_id": u.EditorSet.DefaultEnv3dId}
		useDefaultEnvId = true
	}

	env3d := &model.Env3d{}
	ok, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &optionEnv, env3d)
	if err != nil {
		return nil, err
	}
	if !ok && useDefaultEnvId {
		_, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: repo.CollectionEvn3d}, env3d)
		if err != nil {
			return nil, err
		}
	}
	objUid, _ := primitive.ObjectIDFromHex(uid)

	//获取用户的自定义配置
	//查询对应的环境是否已经存在
	var currTheme *model.EnvTheme
	if u.EditorSet != nil && u.EditorSet.Themes != nil {
		for _, v := range u.EditorSet.Themes {
			if v.Id == env3d.Id.Hex() {
				currTheme = v
				break
			}
		}
	}

	//queentree 的默认环境球
	env3d.Id, _ = primitive.ObjectIDFromHex("628c3433633d10f8e365855f")
	scene := &model.DesignScene{
		Id:        uint64(time.Now().Unix()),
		Name:      shoe.Name,
		Thumbnail: shoe.Thumbnail,
		Images:    []*model.OssType{},
		Videos:    []*model.OssType{},
		Locks:     []string{},
		Background: &model.SceneBackground{
			Type:  env3d.Background.Type,
			Color: env3d.Background.Color,
			Image: env3d.Background.Image,
		},
		Env: &model.DesingSceneEnv3d{
			Id:        env3d.Id,
			Name:      env3d.Name,
			Thumbnail: env3d.Thumbnail,
			Config:    env3d.Config,
			HDR:       env3d.HDR,

			Options: &model.Evn3dOption{
				Rotation: env3d.Options.Rotation,
				Exposure: env3d.Options.Exposure,
			},
			ToneMap: &model.ToneMap{
				Method:     env3d.ToneMap.Method,
				Exposure:   env3d.ToneMap.Exposure,
				Saturation: env3d.ToneMap.Saturation,
				Contrast:   env3d.ToneMap.Contrast,
				Brightness: env3d.ToneMap.Brightness,
			},
		},
		Components: []*model.SceneComponent{},
	}

	if currTheme != nil {
		scene.Env.Options = currTheme.Options
		scene.Env.ToneMap = currTheme.ToneMap
		scene.Background = currTheme.Background
	}

	design := &model.Design{
		UserId:     objUid,
		ShoeId:     shoeid,
		Name:       shoe.Name,
		Sharepwd:   "",
		Thumbnail:  shoe.Thumbnail,
		CreateTime: time.Now(),
		Sex:        shoe.Sex,
		Season:     shoe.Season,
		Type:       shoe.Type,
		IsPublic:   BoolValue(false),
		Scenes:     []*model.DesignScene{scene},
	}

	return design, nil
}

func ServiceDesignCreate(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := struct {
		ShoesId string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}
	if len(body.ShoesId) < 1 {
		return nil, NewError("款式Id不能为空!")
	}

	design, err := createDefaultDesign(apictx, body.ShoesId)
	if err != nil {
		return nil, err
	}
	did, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, design)
	if err != nil {
		return nil, err
	}
	return map[string]string{"id": did}, nil
}

func ServiceDesignList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	uid, _ := primitive.ObjectIDFromHex(apictx.User.ID)
	query["userId"] = uid

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionDesigns,
		Page:        page,
		Size:        size,
		Query:       query,
		Project:     []string{"name", "thumbnail", "createTime", "isPublic"},
		Sort:        bson.M{"createTime": -1},
	})
}

func ServiceDesignPublicList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)
	query["isPublic"] = true

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionDesigns,
		Page:        page,
		Size:        size,
		Query:       query,
		Project:     []string{"name", "thumbnail", "createTime", "isPublic"},
		Sort:        bson.M{"createTime": -1},
	})
}

func ServiceDesignPublic(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	body := &struct {
		IsPublic bool
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}
	return repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionDesigns, id, bson.M{"$set": bson.M{"isPublic": body.IsPublic}})
}

func ServiceDesignCopy(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	scope := c.Query("scope")
	if len(scope) < 1 {
		scope = "company"
	}

	if len(id) < 1 {
		return nil, NewError("id不能为空!")
	}
	did, _ := primitive.ObjectIDFromHex(id)
	uid, _ := primitive.ObjectIDFromHex(apictx.User.ID)

	srcColl := UtilGetDesignCollection(scope)

	option := repo.DocSearchOptions{
		CollectName: srcColl,
		Query:       repo.Map{"_id": did},
	}

	design := &model.Design{}
	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, design)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("项目Id无效!")
	}

	// if apictx.User.ID == design.UserId.Hex() { //当前项目就是用户本身的项目，不进行拷贝
	// 	return id, nil
	// }

	//判断当前项目是否已经被拷贝了
	designSrc := &model.Design{}
	ok, _ = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionDesigns,
		Query:       repo.Map{"fromId": id, "userId": uid},
		Project:     []string{"_id"},
	}, designSrc)

	if ok {
		return designSrc.Id, nil
	}

	design.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
	design.Id = primitive.NilObjectID
	design.IsPublic = BoolValue(false)
	design.CreateTime = time.Now()
	design.FromId = id
	design.State = Int32Value(model.LibState_Created)

	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, design)
}

func ServiceDesignDelete(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")

	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, id)
}

func ServiceDesignDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("id不能为空!")
	}
	scope := c.Query("scope")
	collection := repo.CollectionDesigns
	if scope == "company" || scope == "team" {
		collection = fmt.Sprintf("%s_%s", "design", scope)
	}
	did, _ := primitive.ObjectIDFromHex(id)

	option := repo.DocSearchOptions{
		CollectName: collection,
		Query:       repo.Map{"_id": did},
		Project:     []string{"name", "thumbnail", "editor", "scenes", "shoeId", "createTime", "state", "sharepwd", "isPublic", "sex", "type", "season", "categories"},
	}
	design := &model.Design{}
	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, design)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("项目Id无效!")
	}

	option = repo.DocSearchOptions{
		CollectName: repo.CollectionShoes,
		Query:       repo.Map{"_id": design.ShoeId},
		Project:     []string{"name", "materials", "leftComponents", "meshId"},
	}
	shoe := &model.ShoeMesh{}
	ok, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, shoe)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("shoeId无效!")
	}

	//查询osgjs
	option = repo.DocSearchOptions{
		CollectName: repo.CollectionMesh,
		Query:       repo.Map{"_id": shoe.MeshId},
		Project:     []string{"osgjs", "materials", "file", "shadow"},
	}
	mesh := &model.Mesh{}
	ok, err = repo.RepoSeachDoc(apictx.CreateRepoCtx(), &option, mesh)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, NewError("meshId无效!")
	}

	//获取所有面料
	fabric := []*model.Fabric{}
	fabricMap := map[string]*model.Fabric{}

	//获取所有色卡
	material := []*comm.SpecialMat{}
	materialMap := map[string]*comm.SpecialMat{}

	//env3d
	env3d := []map[string]interface{}{}
	// env3dMap := map[string]bool{}

	//xxx
	for _, item := range design.Scenes {

		//色卡和面料
		for _, comp := range item.Components {
			if comp.MatType == model.MatType_Fabric {
				fid := comp.FabricId.Hex()
				f := fabricMap[fid]
				if f == nil {
					fb := repo.FabricGetById(apictx.CreateRepoCtx(), comp.FabricScope, fid, []string{"name", "colorCards"})
					if fb != nil {
						fabricMap[fid] = fb
						fabric = append(fabric, fb)
					}
					f = fb
				}

				if comp.Card == nil && f != nil {
					currCardId := comp.CardId.Hex()
					for _, item := range f.ColorCards {
						if item.MatId.Hex() == currCardId {
							comp.Card = item
							break
						}
					}
				}
			}

			if comp.MatType == model.MatType_Effect {
				fid := comp.CardId.Hex()
				f := materialMap[fid]
				if f == nil {
					mat := repo.SpecialMatGetById(apictx.CreateRepoCtx(), fid)
					if mat != nil {
						materialMap[fid] = mat
						material = append(material, mat)
					}
					f = mat
				}
				if f != nil && comp.Card == nil {
					comp.Card = f.Copy2MaterialConf()
				}
			}
		}
	}

	return map[string]interface{}{
		"env3d":    env3d,
		"fabric":   fabric,
		"material": material,
		"design":   design,
		"shoe":     shoe,
		"mesh":     mesh,
	}, nil
}

func ServiceDesignResetShoe(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &struct {
		Id     string
		ShoeId string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}
	if len(body.Id) < 1 || len(body.ShoeId) < 1 {
		return nil, NewError("Id和ShoeId不能为空")
	}

	design, err := createDefaultDesign(apictx, body.ShoeId)
	if err != nil {
		return nil, err
	}
	design.State = Int32Value(model.LibState_Created)

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, body.Id, design)
}

func ServiceDesignResetDesign(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &struct {
		CurrId    string
		CurrScope string
		ToId      string
		ToScope   string
	}{}

	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}
	if len(body.CurrId) < 1 || len(body.ToId) < 1 || len(body.CurrScope) < 1 || len(body.ToScope) < 1 {
		return nil, NewError("参数不合法！")
	}
	if body.CurrId == body.ToId {
		return nil, NewError("两个款式相同不能重置!")
	}

	toColl := UtilGetDesignCollection(body.ToScope)

	//查询目标设计
	design := &model.Design{}
	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: toColl,
		Query:       repo.Map{"_id": body.ToId},
	}, design)

	if err != nil || !ok {
		return nil, NewError("目标款式查询失败!")
	}

	// design.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
	design.UserId = primitive.NilObjectID
	design.Id = primitive.NilObjectID
	design.Sharepwd = ""
	design.CreateTime = time.Now()
	design.IsPublic = nil
	design.Categories = []string{}
	design.Name = ""
	design.TeamId = ""    //团队Id 不重置
	design.CompanyId = "" //企业Id 不重置

	currColl := UtilGetDesignCollection(body.CurrScope)

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), currColl, body.CurrId, design)
}

func ServiceDesignUpdate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("Id不能为空")
	}
	body := &model.Design{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}

	body.Id = primitive.NilObjectID
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, id, body)
}

func ServiceDesignCreateScene(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("id不能为空!")
	}
	did, _ := primitive.ObjectIDFromHex(id)

	fmt.Println(did)

	scene := model.DesignScene{}
	err := c.ShouldBindJSON(&scene)
	if err != nil {
		return nil, err
	}

	scope := c.Param("scope")
	coll := repo.CollectionDesigns
	if scope == "team" || scope == "company" {
		coll = fmt.Sprintf("%s_%s", "design", scope)
	}

	return repo.RepoDocArrayAppend(apictx.CreateRepoCtx(), coll, id, "scenes", scene)
}

func ServiceDesignUpdateScene(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("id不能为空!")
	}

	scope := c.Param("scope")

	scene := model.DesignScene{}
	err := c.ShouldBindJSON(&scene)
	if err != nil {
		return nil, err
	}
	if scene.Id < 1 {
		return nil, NewError("场景Id不能为空!")
	}

	optSet := repo.Map{}
	if len(scene.Name) > 0 {
		optSet["scenes.$.name"] = scene.Name
	}
	if scene.Thumbnail != nil {
		optSet["scenes.$.thumbnail"] = scene.Thumbnail
	}
	if scene.Background != nil {
		optSet["scenes.$.background"] = scene.Background
	}

	if len(scene.Locks) > 0 {
		optSet["scenes.$.locks"] = scene.Locks
	}
	if scene.Env != nil {
		optSet["scenes.$.env"] = scene.Env
	}
	if len(scene.Components) > 0 {
		optSet["scenes.$.components"] = scene.Components
	}
	if len(scene.Models) > 0 {
		optSet["scenes.$.models"] = scene.Models
	}

	if len(scene.Stickers) > 0 {
		optSet["scenes.$.stickers"] = scene.Stickers
	}

	if scene.Lights != nil {
		optSet["scenes.$.lights"] = scene.Lights
	}

	coll := repo.CollectionDesigns
	if scope == "team" || scope == "company" {
		coll = fmt.Sprintf("%s_%s", "design", scope)
	}

	option := &repo.ArrayOneUpdateOption{
		CollectName: coll,
		Id:          id,
		Query:       repo.Map{"scenes.id": scene.Id},
		Set:         optSet,
	}

	ret, err := repo.RepoDocArrayOneUpdate(apictx.CreateRepoCtx(), option)

	return ret, err
}

func ServiceDesignRomoveScene(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("id不能为空!")
	}

	scene := struct {
		SceneId uint64
	}{}

	err := c.ShouldBindJSON(&scene)
	if err != nil {
		return nil, err
	}

	scope := c.Param("scope")
	coll := repo.CollectionDesigns
	if scope == "team" || scope == "company" {
		coll = fmt.Sprintf("%s_%s", "design", scope)
	}

	option := &repo.ArrayOneRemoveOption{
		CollectName: coll,
		Id:          id,
		ArrayQuery:  repo.Map{"scenes": bson.M{"id": scene.SceneId}},
	}
	ret, err := repo.RepoDocArrayOneRemove(apictx.CreateRepoCtx(), option)
	if ret.ModifiedCount != 1 {
		return nil, NewError("删除失败")
	}
	return ret, err
}

func ServiceDesignRename(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("id不能为空!")
	}
	body := struct {
		Name string
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, id, bson.M{"name": body.Name})
}

func UtilGetDesignCollection(scope string) string {
	if scope == "team" || scope == "company" {
		return fmt.Sprintf("%s_%s", "design", scope)
	}
	return repo.CollectionDesigns
}

func UtilGetFabricCollection(scope string) string {
	if scope == "team" || scope == "company" {
		return fmt.Sprintf("%s_%s", "fabric", scope)
	} else if scope == "user" {
		return repo.CollectionFabric
	}
	return "fabric_company"
}

func UtilGetImgMatCollection(scope string) string {
	if scope == "team" || scope == "company" {
		return fmt.Sprintf("%s_%s", "imgmat", scope)
	}
	return repo.CollectionImageMat
}

func ServiceDesignSceneThumbnail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("id不能为空!")
	}
	body := struct {
		Scope     string
		SceneId   uint64
		Thumbnail *model.OssType
	}{}
	err := c.ShouldBindJSON(&body)
	if err != nil {
		return nil, err
	}

	//更新项目封面
	//repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, id, bson.M{"thumbnail": body.Thumbnail})

	//更新场景封面
	option := &repo.ArrayOneUpdateOption{
		CollectName: UtilGetDesignCollection(body.Scope),
		Id:          id,
		Query:       repo.Map{"scenes.id": body.SceneId},
		Set:         repo.Map{"scenes.$.thumbnail": body.Thumbnail},
	}

	ret, err := repo.RepoDocArrayOneUpdate(apictx.CreateRepoCtx(), option)
	if ret.ModifiedCount != 1 {
		return nil, NewError("场景封面更新失败!")
	}
	return ret, err
}
